use crate::{
    app_error::AppError,
    pe_parse::pe::PE,
    services::{self, file::*, GLOBAL_FILE_DATA},
};
use serde::Serialize;
use tauri::{AppHandle, Manager};

#[derive(Serialize)]
pub struct PeNodeTreeData {
    title: String,
    key: String,
    children: Vec<PeNodeTreeData>,
}
// set_complete
#[tauri::command]
pub fn set_complete(app: AppHandle) -> Result<(), AppError> {
    let splash_window = app.get_webview_window("splashscreen");
    if let Some(splash_window) = splash_window {
        splash_window.close().unwrap();
    }
    let main_window = app.get_webview_window("main");
    if let Some(main_window) = main_window {
        main_window.show().unwrap();
    }
    Ok(())
}

#[tauri::command]
pub fn command_get_file_pe_node_tree_data() -> Result<Vec<PeNodeTreeData>, AppError> {
    // 1. 如果没有打开文件，则返回错误
    if GLOBAL_FILE_DATA.lock().unwrap().is_none() {
        return Err(AppError::NoFileOpened);
    }
    // 从文件路径中获取文件名
    let file_name: &str = unsafe {
        let file_path = services::GLOBAL_FILE_PATH.as_ref().unwrap();
        let file_name = std::path::Path::new(file_path).file_name().unwrap();
        file_name.to_str().unwrap()
    };
    let result = PeNodeTreeData {
        title: format!("文件: {}", file_name),
        key: "file_name".to_string(),
        children: vec![
            PeNodeTreeData {
                title: "DOS 头部".to_string(),
                key: "dos_header".to_string(),
                children: vec![], // 空数组表示没有子节点
            },
            PeNodeTreeData {
                title: "NT 头部".to_string(),
                key: "nt_header".to_string(),
                children: vec![
                    PeNodeTreeData {
                        title: "文件头部".to_string(),
                        key: "file_header".to_string(),
                        children: vec![],
                    },
                    PeNodeTreeData {
                        title: "可选头部".to_string(),
                        key: "optional_header".to_string(),
                        children: vec![PeNodeTreeData {
                            title: "数据目录".to_string(),
                            key: "data_directories".to_string(),
                            children: vec![],
                        }],
                    },
                ],
            },
            PeNodeTreeData {
                title: "节区头部".to_string(),
                key: "section_header".to_string(),
                children: vec![],
            },
        ],
    };
    Ok(vec![result])
}

// 命令，打开文件
#[tauri::command]
pub fn command_open_file(file_path: &str) -> Result<(), AppError> {
    let mmap = Some(services::file::mmap_mut_file(file_path)?);
    // 将内存clone到Vec中
    let mmap = mmap.as_ref().unwrap();
    let mmap = mmap.to_vec();
    *services::GLOBAL_FILE_DATA.lock().unwrap() = Some(services::PeData(mmap));
    unsafe {
        services::GLOBAL_FILE_PATH = Some(file_path.to_string());
    }

    // 判断文件是否是64位
    let binding = GLOBAL_FILE_DATA.lock().unwrap();
    let file_data = binding.as_ref().unwrap();
    let is64bit = file_data.is_64_bit()?;
    unsafe {
        services::GLOBAL_IS_64_BIT = Some(is64bit);
    }
    Ok(())
}

// 命令，获取DOS头数据
#[tauri::command]
pub fn command_get_pe_data_dos_header() -> Result<ResponseDOSHeaderData, AppError> {
    let binding = GLOBAL_FILE_DATA.lock().unwrap();
    let file_data = binding.as_ref().unwrap();
    let dos_header = file_data.get_dos_header()?;
    let result = ResponseDOSHeaderData {
        fields: dos_header.clone(),
        base_offset: 0,
    };
    Ok(result)
}

// 命令，获取NT头数据
#[tauri::command]
pub fn command_get_pe_data_nt_header() -> Result<ResponseNTHeaderData, AppError> {
    let result = get_nt_headers_data()?;
    Ok(result)
}

// 命令，获取文件头数据
#[tauri::command]
pub fn command_get_pe_data_file_header() -> Result<ResponseFileHeaderData, AppError> {
    let result = services::file::get_file_header_data()?;
    Ok(result)
}

// 命令，获取可选头数据
#[tauri::command]
pub fn command_get_pe_data_optional_header() -> Result<ResponseOptionalHeaderData, AppError> {
    let result = services::file::get_optional_header_data()?;
    Ok(result)
}
