use super::{error::PEParseError, header::*};
use std::{ops::{Deref, DerefMut}, option};

pub trait PE: Deref<Target = [u8]> + DerefMut<Target = [u8]> + Sized {
    /// Get a reference to a type at a given offset.
    fn get_ref<T>(&self, offset: usize) -> Result<&T, PEParseError> {
        let len = std::mem::size_of::<T>();
        if offset + len > self.len() {
            return Err(PEParseError::OutOfBounds);
        }
        let ptr = self.as_ptr().wrapping_offset(offset as isize) as *const T;
        Ok(unsafe { &*ptr })
    }

    /// Get a mutable reference to a type at a given offset.
    fn get_mut<T>(&mut self, offset: usize) -> Result<&mut T, PEParseError> {
        let len = std::mem::size_of::<T>();
        if offset + len > self.len() {
            return Err(PEParseError::OutOfBounds);
        }
        let ptr = self.as_mut_ptr().wrapping_offset(offset as isize) as *mut T;
        Ok(unsafe { &mut *ptr })
    }

    // 判断是64位还是32位
    fn is_64_bit(&self) -> Result<bool, PEParseError> {
        // 先以32位加载可选头，通过可选头的Magic字段判断
        let optional_header: &ImageOptionalHeader32 = self.get_optional_header_32()?;
        let is_64_bit = optional_header.magic == 0x20b;
        Ok(is_64_bit)
    }

    

    /// Get the DOS header without verifying its contents.
    fn get_dos_header(&self) -> Result<&ImageDosHeader, PEParseError> {
        let result = self.get_ref::<ImageDosHeader>(0)?;
        Ok(result)
    }
    /// Get the DOS header without mutating its contents.
    fn get_dos_header_mut(&mut self) -> Result<&mut ImageDosHeader, PEParseError> {
        let result = self.get_mut::<ImageDosHeader>(0)?;
        Ok(result)
    }

    

    fn get_nt_headers32(&self) -> Result<&ImageNTHeaders32, PEParseError> {
        let nt_offset = self.get_nt_headers_offset()?;
        let result = self.get_ref::<ImageNTHeaders32>(nt_offset)?;
        Ok(result)
    }
    fn get_nt_headers64(&self) -> Result<&ImageNTHeaders64, PEParseError> {
        let nt_offset = self.get_nt_headers_offset()?;
        let result = self.get_ref::<ImageNTHeaders64>(nt_offset)?;
        Ok(result)
    }
    fn get_nt_headers(&self) -> Result<ImageNTHeaders, PEParseError> {
        let is_64_bit = self.is_64_bit()?;
        let result = match is_64_bit {
            true => ImageNTHeaders::NTHeaders64(*self.get_nt_headers64()?),
            false => ImageNTHeaders::NTHeaders32(*self.get_nt_headers32()?),
        };
        Ok(result)
    }

    // 获取nt头的偏移
    fn get_nt_headers_offset(&self) -> Result<usize, PEParseError> {
        let dos_header = self.get_dos_header()?;
        let nt_offset = dos_header.e_lfanew.0 as usize;
        Ok(nt_offset)
    }

    /// Get the NT headers without mutating its contents.
    fn get_nt_headers_mut(&mut self) -> Result<&mut ImageNTHeaders32, PEParseError> {
        let nt_offset = self.get_nt_headers_offset()?;
        let result = self.get_mut::<ImageNTHeaders32>(nt_offset)?;
        Ok(result)
    }

    // 获取文件头的偏移
    fn get_file_header_offset(&self) -> Result<usize, PEParseError> {
        // 1. 获取nt头偏移
        let nt_offset = self.get_nt_headers_offset()?;
        // 2. 计算文件头偏移
        let file_header_offset = nt_offset + std::mem::size_of::<u32>();
        Ok(file_header_offset)
    }

    // 获取文件头数据
    fn get_file_header(&self) -> Result<&ImageFileHeader, PEParseError> {
        let file_header_offset = self.get_file_header_offset()?;
        let result = self.get_ref::<ImageFileHeader>(file_header_offset)?;
        Ok(result)
    }
    // 获取文件头数据
    fn get_file_header_mut(&mut self) -> Result<&mut ImageFileHeader, PEParseError> {
        let file_header_offset = self.get_file_header_offset()?;
        let result = self.get_mut::<ImageFileHeader>(file_header_offset)?;
        Ok(result)
    }

    // 获取节区数量
    fn get_number_of_sections(&self) -> Result<usize, PEParseError> {
        let file_header = self.get_file_header()?;
        Ok(file_header.number_of_sections as usize)
    }

    // 获取可选头大小
    fn get_size_of_optional_header(&self) -> Result<usize, PEParseError> {
        let file_header = self.get_file_header()?;
        Ok(file_header.size_of_optional_header as usize)
    }

    fn get_optional_header_32(&self) -> Result<&ImageOptionalHeader32, PEParseError> {
        let file_header_offset = self.get_file_header_offset()?;
        let optional_header_offset = file_header_offset + std::mem::size_of::<ImageFileHeader>();
        let result = self.get_ref::<ImageOptionalHeader32>(optional_header_offset)?;
        Ok(result)
    }

    // 获取可选头
    fn get_optional_header_64(&self) -> Result<&ImageOptionalHeader64, PEParseError> {
        let file_header_offset = self.get_file_header_offset()?;
        let optional_header_offset = file_header_offset + std::mem::size_of::<ImageFileHeader>();
        let result = self.get_ref::<ImageOptionalHeader64>(optional_header_offset)?;
        Ok(result)
    }
    
    fn get_optional_header(&self) -> Result<ImageOptionalHeader, PEParseError> {
        let is_64_bit = self.is_64_bit()?;
        let result = match is_64_bit {
            true => ImageOptionalHeader::OptionalHeader64(*self.get_optional_header_64()?),
            false => ImageOptionalHeader::OptionalHeader32(*self.get_optional_header_32()?),
        };
        Ok(result)
    }

    fn get_optional_header_offset(&self) -> Result<usize, PEParseError> {
        let file_header_offset = self.get_file_header_offset()?;
        let optional_header_offset = file_header_offset + std::mem::size_of::<ImageFileHeader>();
        Ok(optional_header_offset)
    }

    // 获取所有节区头数据
    fn get_image_section_headers(&self) -> Result<&[ImageSectionHeader], PEParseError> {
        // 1. 获取节区数量
        let number_of_sections = self.get_number_of_sections()?;
        // 2. 找到节区头的偏移
        // 节区头偏移在可选头之后，可选头大小是可变的，所以需要计算
        let optional_header_size = self.get_size_of_optional_header()?;
        let file_header_offset = self.get_file_header_offset()?;
        let section_header_offset = file_header_offset + std::mem::size_of::<ImageFileHeader>() + optional_header_size;
        // 3. 获取节区头数据
        unsafe {
            let ptr = self.as_ptr().wrapping_offset(section_header_offset as isize) as *const ImageSectionHeader;
            let result = std::slice::from_raw_parts(ptr, number_of_sections);
            Ok(result)
        }
    }
}
